@@ -40,15 +40,36 @@ namespace Mono.Profiler {
 	public interface ILoadedMethod<LC> : ILoadedElement where LC : ILoadedClass {
 		LC Class {get;}
 	}
-	public interface IUnmanagedFunctionFromID<MR,UFR> : ILoadedElement where UFR : IUnmanagedFunctionFromRegion where MR : IExecutableMemoryRegion<UFR> {
+	public interface IUnmanagedFunctionFromID<MR,UFR> : ILoadedElement where UFR : IUnmanagedFunctionFromRegion<UFR> where MR : IExecutableMemoryRegion<UFR> {
 		MR Region {get;}
 	}
-	public interface IUnmanagedFunctionFromRegion {
+	public interface IUnmanagedFunctionFromRegion<UFR> where UFR : IUnmanagedFunctionFromRegion<UFR> {
+		IExecutableMemoryRegion<UFR> Region {get; set;}
 		string Name {get; set;}
 		uint StartOffset {get; set;}
 		uint EndOffset {get; set;}
 	}
-	public interface IExecutableMemoryRegion<UFR> : ILoadedElement where UFR : IUnmanagedFunctionFromRegion {
+	public struct StackSectionElement<LC,LM> where LC : ILoadedClass where LM : ILoadedMethod<LC> {
+		LM method;
+		public LM Method {
+			get {
+				return method;
+			}
+			set {
+				method = value;
+			}
+		}
+		bool isBeingJitted;
+		public bool IsBeingJitted {
+			get {
+				return isBeingJitted;
+			}
+			set {
+				isBeingJitted = value;
+			}
+		}
+	}
+	public interface IExecutableMemoryRegion<UFR> : ILoadedElement where UFR : IUnmanagedFunctionFromRegion<UFR> {
 		ulong StartAddress {get;}
 		ulong EndAddress {get;}
 		uint FileOffset {get;}
@@ -57,35 +78,39 @@ namespace Mono.Profiler {
 		UFR[] Functions {get;}
 		void SortFunctions ();
 	}
-	public interface IHeapObject<HO,LC> where HO: IHeapObject<HO,LC> where LC : ILoadedClass {
+	public interface IAllocatedObject<LC> where LC : ILoadedClass {
 		ulong ID {get;}
 		LC Class {get;}
 		uint Size {get;}
+	}
+	public interface IHeapObject<HO,LC> : IAllocatedObject<LC> where HO: IHeapObject<HO,LC> where LC : ILoadedClass {
 		HO[] References {get;}
 		HO[] BackReferences {get;}
 	}
+	public delegate HO HeapObjectFactory<HO,LC> (ulong ID) where HO : IHeapObject<HO,LC> where LC : ILoadedClass;
 	public interface IHeapSnapshot<HO,LC> where HO: IHeapObject<HO,LC> where LC : ILoadedClass {
 		uint Collection {get;}
 		ulong StartCounter {get;}
 		DateTime StartTime {get;}
 		ulong EndCounter {get;}
 		DateTime EndTime {get;}
+		TimeSpan HeaderStartTime {get;}
 		HO NewHeapObject (ulong id, LC c, uint size, ulong[] referenceIds, int referencesCount);
 		HO GetHeapObject (ulong id);
 		HO[] HeapObjects {get;}
 		bool RecordSnapshot {get;}
 	}
 	
-	public interface ILoadedElementFactory<LC,LM,UFR,UFI,MR,HO,HS> where LC : ILoadedClass where LM : ILoadedMethod<LC> where UFR : IUnmanagedFunctionFromRegion where UFI : IUnmanagedFunctionFromID<MR,UFR> where MR : IExecutableMemoryRegion<UFR> where HO: IHeapObject<HO,LC> where HS: IHeapSnapshot<HO,LC> {
+	public interface ILoadedElementFactory<LC,LM,UFR,UFI,MR,HO,HS> where LC : ILoadedClass where LM : ILoadedMethod<LC> where UFR : IUnmanagedFunctionFromRegion<UFR> where UFI : IUnmanagedFunctionFromID<MR,UFR> where MR : IExecutableMemoryRegion<UFR> where HO: IHeapObject<HO,LC> where HS: IHeapSnapshot<HO,LC> {
 		LC NewClass (uint id, string name, uint size);
 		LM NewMethod (uint id, LC c, string name);
 		MR NewExecutableMemoryRegion (uint id, string fileName, uint fileOffset, ulong startAddress, ulong endAddress);
 		UFI NewUnmanagedFunction (uint id, string name, MR region);
-		HS NewHeapSnapshot (uint collection, ulong startCounter, DateTime startTime, ulong endCounter, DateTime endTime, LC[] initialAllocations, bool recordSnapshot);
+		HS NewHeapSnapshot (uint collection, ulong startCounter, DateTime startTime, ulong endCounter, DateTime endTime, TimeSpan headerStartTime, LC[] initialAllocations, bool recordSnapshot);
 		bool RecordHeapSnapshots {get; set;}
 	}
 	
-	public interface ILoadedElementHandler<LC,LM,UFR,UFI,MR,HO,HS> : ILoadedElementFactory<LC,LM,UFR,UFI,MR,HO,HS> where LC : ILoadedClass where LM : ILoadedMethod<LC> where UFR : IUnmanagedFunctionFromRegion where UFI : IUnmanagedFunctionFromID<MR,UFR> where MR : IExecutableMemoryRegion<UFR> where HO: IHeapObject<HO,LC> where HS: IHeapSnapshot<HO,LC> {
+	public interface ILoadedElementHandler<LC,LM,UFR,UFI,MR,HO,HS> : ILoadedElementFactory<LC,LM,UFR,UFI,MR,HO,HS> where LC : ILoadedClass where LM : ILoadedMethod<LC> where UFR : IUnmanagedFunctionFromRegion<UFR> where UFI : IUnmanagedFunctionFromID<MR,UFR> where MR : IExecutableMemoryRegion<UFR> where HO: IHeapObject<HO,LC> where HS: IHeapSnapshot<HO,LC> {
 		LC[] Classes {get;}
 		LC GetClass (uint id);
 		LM[] Methods {get;}
@@ -99,10 +124,12 @@ namespace Mono.Profiler {
 		UFI GetUnmanagedFunctionByID (uint id);
 		HS[] HeapSnapshots {get;}
 	}
-	public interface IProfilerEventHandler<LC,LM,UFR,UFI,MR,EH,HO,HS> where LC : ILoadedClass where LM : ILoadedMethod<LC> where UFR : IUnmanagedFunctionFromRegion where UFI : IUnmanagedFunctionFromID<MR,UFR> where MR : IExecutableMemoryRegion<UFR> where EH : ILoadedElementHandler<LC,LM,UFR,UFI,MR,HO,HS> where HO: IHeapObject<HO,LC> where HS: IHeapSnapshot<HO,LC> {
+	public interface IProfilerEventHandler<LC,LM,UFR,UFI,MR,EH,HO,HS> where LC : ILoadedClass where LM : ILoadedMethod<LC> where UFR : IUnmanagedFunctionFromRegion<UFR> where UFI : IUnmanagedFunctionFromID<MR,UFR> where MR : IExecutableMemoryRegion<UFR> where EH : ILoadedElementHandler<LC,LM,UFR,UFI,MR,HO,HS> where HO: IHeapObject<HO,LC> where HS: IHeapSnapshot<HO,LC> {
 		DirectivesHandler Directives {get;}
 		
 		EH LoadedElements {get;}
+		Double TicksPerCounterUnit {get;}
+		TimeSpan ClicksToTimeSpan (ulong clicks);
 		
 		void Start (uint version, string runtimeFile, ProfilerFlags flags, ulong startCounter, DateTime startTime);
 		void End (uint version, ulong endCounter, DateTime endTime);
@@ -124,7 +151,7 @@ namespace Mono.Profiler {
 		void ClassStartUnload (LC c, ulong counter);
 		void ClassEndUnload (LC c, ulong counter);
 		
-		void Allocation (LC c, uint size, LM caller, bool jitTime, ulong counter);
+		void Allocation (LC c, uint size, LM caller, bool jitTime, ulong objectId, ulong counter);
 		void Exception (LC c, ulong counter);
 		
 		void MethodEnter (LM m, ulong counter);
@@ -133,6 +160,8 @@ namespace Mono.Profiler {
 		void MethodJitEnd (LM m, ulong counter, bool success);
 		void MethodFreed (LM m, ulong counter);
 		
+		void AdjustStack (uint lastValidFrame, uint topSectionSize, StackSectionElement<LC,LM>[] topSection);
+		
 		void MethodStatisticalHit (LM m);
 		void UnknownMethodStatisticalHit ();
 		void UnmanagedFunctionStatisticalHit (UFR f);
@@ -164,6 +193,9 @@ namespace Mono.Profiler {
 		void AllocationSummaryStart (uint collection, ulong startCounter, DateTime startTime);
 		void ClassAllocationSummary (LC c, uint reachableInstances, uint reachableBytes, uint unreachableInstances, uint unreachableBytes);
 		void AllocationSummaryEnd (uint collection, ulong endCounter, DateTime endTime);
+		
+		void InitializeData (byte[] data, uint currentOffset);
+		void DataProcessed (uint offset);
 	}
 	
 	public class BaseLoadedElement : ILoadedElement {
@@ -210,9 +242,9 @@ namespace Mono.Profiler {
 			this.c = c;
 		}
 	}
-	public class BaseUnmanagedFunctionFromRegion : IUnmanagedFunctionFromRegion {
-		IExecutableMemoryRegion<IUnmanagedFunctionFromRegion> region;
-		public IExecutableMemoryRegion<IUnmanagedFunctionFromRegion> Region {
+	public class BaseUnmanagedFunctionFromRegion<UFR> : IUnmanagedFunctionFromRegion<UFR> where UFR : IUnmanagedFunctionFromRegion<UFR> {
+		IExecutableMemoryRegion<UFR> region;
+		public IExecutableMemoryRegion<UFR> Region {
 			get {
 				return region;
 			}
@@ -253,14 +285,14 @@ namespace Mono.Profiler {
 			this.startOffset = 0;
 			this.endOffset = 0;
 		}
-		public BaseUnmanagedFunctionFromRegion (IExecutableMemoryRegion<IUnmanagedFunctionFromRegion> region, string name, uint startOffset, uint endOffset) {
+		public BaseUnmanagedFunctionFromRegion (IExecutableMemoryRegion<UFR> region, string name, uint startOffset, uint endOffset) {
 			this.region = region;
 			this.name = name;
 			this.startOffset = startOffset;
 			this.endOffset = endOffset;
 		}
 	}
-	public class BaseUnmanagedFunctionFromID<MR,UFR>: BaseLoadedElement, IUnmanagedFunctionFromID<MR,UFR>  where UFR : IUnmanagedFunctionFromRegion where MR : IExecutableMemoryRegion<UFR> {
+	public class BaseUnmanagedFunctionFromID<MR,UFR>: BaseLoadedElement, IUnmanagedFunctionFromID<MR,UFR>  where UFR : IUnmanagedFunctionFromRegion<UFR> where MR : IExecutableMemoryRegion<UFR> {
 		MR region;
 		public MR Region {
 			get {
@@ -273,7 +305,7 @@ namespace Mono.Profiler {
 		}
 	}
 	
-	public class HeapObject<LC> : IHeapObject<HeapObject<LC>,LC> where LC : ILoadedClass {
+	public class BaseHeapObject<HO,LC> : IHeapObject<HO,LC> where LC : ILoadedClass where HO : BaseHeapObject<HO,LC> {
 		ulong id;
 		public ulong ID {
 			get {
@@ -300,15 +332,15 @@ namespace Mono.Profiler {
 			}
 		}
 		
-		static HeapObject<LC>[] emptyReferences = new HeapObject<LC> [0];
-		public static HeapObject<LC>[] EmptyReferences {
+		static HO[] emptyReferences = new HO [0];
+		public static HO[] EmptyReferences {
 			get {
 				return emptyReferences;
 			}
 		}
 		
-		HeapObject<LC>[] references;
-		public HeapObject<LC>[] References {
+		HO[] references;
+		public HO[] References {
 			get {
 				return references;
 			}
@@ -317,8 +349,8 @@ namespace Mono.Profiler {
 			}
 		}
 		
-		HeapObject<LC>[] backReferences;
-		public HeapObject<LC>[] BackReferences {
+		HO[] backReferences;
+		public HO[] BackReferences {
 			get {
 				return backReferences;
 			}
@@ -334,16 +366,16 @@ namespace Mono.Profiler {
 		internal void AllocateBackReferences () {
 			if (references != null) {
 				int referencesCount = 0;
-				foreach (HeapObject<LC> reference in references) {
+				foreach (HO reference in references) {
 					if (reference != null) {
 						referencesCount ++;
 					}
 				}
 				if (referencesCount != references.Length) {
 					if (referencesCount > 0) {
-						HeapObject<LC>[] newReferences = new HeapObject<LC> [referencesCount];
+						HO[] newReferences = new HO [referencesCount];
 						referencesCount = 0;
-						foreach (HeapObject<LC> reference in references) {
+						foreach (HO reference in references) {
 							if (reference != null) {
 								newReferences [referencesCount] = reference;
 								referencesCount ++;
@@ -359,26 +391,18 @@ namespace Mono.Profiler {
 			}
 			
 			if (backReferencesCounter > 0) {
-				backReferences = new HeapObject<LC> [backReferencesCounter];
+				backReferences = new HO [backReferencesCounter];
 				backReferencesCounter = 0;
 			} else {
-				references = emptyReferences;
+				backReferences = emptyReferences;
 			}
 		}
-		internal void AddBackReference (HeapObject<LC> heapObject) {
+		internal void AddBackReference (HO heapObject) {
 			backReferences [backReferencesCounter] = heapObject;
 			backReferencesCounter ++;
 		}
 		
-		public HeapObject (ulong id, LC c, uint size, HeapObject<LC>[] references) {
-			this.id = id;
-			this.c = c;
-			this.size = size;
-			this.references = references;
-			this.backReferences = null;
-			this.backReferencesCounter = 0;
-		}
-		public HeapObject (ulong id) {
+		public BaseHeapObject (ulong id) {
 			this.id = id;
 			this.c = default(LC);
 			this.size = 0;
@@ -388,9 +412,10 @@ namespace Mono.Profiler {
 		}
 	}
 	
-	public abstract class BaseHeapSnapshot<HO,LC> : IHeapSnapshot<HeapObject<LC>,LC> where LC : ILoadedClass {
-		Dictionary<ulong,HeapObject<LC>> heap;
+	public abstract class BaseHeapSnapshot<HO,LC> : IHeapSnapshot<HO,LC> where LC : ILoadedClass where HO : BaseHeapObject<HO,LC> {
+		Dictionary<ulong,HO> heap;
 		bool backReferencesInitialized;
+		HeapObjectFactory<HO,LC> heapObjectFactory;
 		
 		uint collection;
 		public uint Collection {
@@ -423,15 +448,21 @@ namespace Mono.Profiler {
 				return endTime;
 			}
 		}
-		
-		public HeapObject<LC> NewHeapObject (ulong id, LC c, uint size, ulong[] referenceIds, int referencesCount) {
+		TimeSpan headerStartTime;
+		public TimeSpan HeaderStartTime {
+			get {
+				return headerStartTime;
+			}
+		}
+			
+		public HO NewHeapObject (ulong id, LC c, uint size, ulong[] referenceIds, int referencesCount) {
 			if (backReferencesInitialized) {
 				throw new Exception ("Cannot create heap objects after backReferencesInitialized is true");
 			}
 			
 			if (recordSnapshot) {
-				HeapObject<LC>[] references = new HeapObject<LC>[referencesCount];
-				HeapObject<LC> result = GetOrCreateHeapObject (id);
+				HO[] references = new HO [referencesCount];
+				HO result = GetOrCreateHeapObject (id);
 				for (int i = 0; i < references.Length; i++) {
 					references [i] = GetOrCreateHeapObject (referenceIds [i]);
 					references [i].IncrementBackReferences ();
@@ -451,9 +482,9 @@ namespace Mono.Profiler {
 			}
 			
 			//FIXME: Bad objects should not happen anymore...
-			Dictionary<ulong,HeapObject<LC>> badObjects = new Dictionary<ulong,HeapObject<LC>> ();
+			Dictionary<ulong,HO> badObjects = new Dictionary<ulong,HO> ();
 			
-			foreach (HeapObject<LC> heapObject in heap.Values) {
+			foreach (HO heapObject in heap.Values) {
 				if (heapObject.Class != null) {
 					heapObject.AllocateBackReferences ();
 				} else {
@@ -465,8 +496,8 @@ namespace Mono.Profiler {
 				heap.Remove (id);
 			}
 			
-			foreach (HeapObject<LC> heapObject in heap.Values) {
-				foreach (HeapObject<LC> reference in heapObject.References) {
+			foreach (HO heapObject in heap.Values) {
+				foreach (HO reference in heapObject.References) {
 					reference.AddBackReference (heapObject);
 				}
 			}
@@ -474,12 +505,12 @@ namespace Mono.Profiler {
 			backReferencesInitialized = true;
 		}
 		
-		HeapObject<LC> GetOrCreateHeapObject (ulong id) {
+		HO GetOrCreateHeapObject (ulong id) {
 			if (recordSnapshot) {
 				if (heap.ContainsKey (id)) {
 					return heap [id];
 				} else {
-					HeapObject<LC> result = new HeapObject<LC> (id);
+					HO result = heapObjectFactory (id);
 					heap [id] = result;
 					return result;
 				}
@@ -488,13 +519,13 @@ namespace Mono.Profiler {
 			}
 		}
 		
-		public HeapObject<LC> GetHeapObject (ulong id) {
+		public HO GetHeapObject (ulong id) {
 			return heap [id];
 		}
 		
-		public HeapObject<LC>[] HeapObjects {
+		public HO[] HeapObjects {
 			get {
-				HeapObject<LC>[] result = new HeapObject<LC> [heap.Values.Count];
+				HO[] result = new HO [heap.Values.Count];
 				heap.Values.CopyTo (result, 0);
 				return result;
 			}
@@ -507,14 +538,16 @@ namespace Mono.Profiler {
 			}
 		}
 		
-		public BaseHeapSnapshot (uint collection, ulong startCounter, DateTime startTime, ulong endCounter, DateTime endTime, bool recordSnapshot) {
+		public BaseHeapSnapshot (HeapObjectFactory<HO,LC> heapObjectFactory, uint collection, ulong startCounter, DateTime startTime, ulong endCounter, DateTime endTime, TimeSpan headerStartTime, bool recordSnapshot) {
+			this.heapObjectFactory = heapObjectFactory;
 			this.collection = collection;
 			this.startCounter = startCounter;
 			this.startTime = startTime;
 			this.endCounter = endCounter;
 			this.endTime = endTime;
+			this.headerStartTime = headerStartTime;
 			this.recordSnapshot = recordSnapshot;
-			heap = new Dictionary<ulong,HeapObject<LC>> ();
+			heap = new Dictionary<ulong,HO> ();
 			backReferencesInitialized = false;
 		}
 	}
@@ -632,6 +665,8 @@ namespace Mono.Profiler {
 	public enum DirectiveCodes {
 		END = 0,
 		ALLOCATIONS_CARRY_CALLER = 1,
+		ALLOCATIONS_HAVE_STACK = 2,
+		ALLOCATIONS_CARRY_ID = 3,
 		LAST
 	}
 	
@@ -646,13 +681,35 @@ namespace Mono.Profiler {
 			allocationsCarryCallerMethod = true;
 		}
 		
+		bool allocationsHaveStackTrace;
+		public bool AllocationsHaveStackTrace {
+			get {
+				return allocationsHaveStackTrace;
+			}
+		}
+		public void AllocationsHaveStackTraceReceived () {
+			allocationsHaveStackTrace = true;
+		}
+		
+		bool allocationsCarryId;
+		public bool AllocationsCarryId {
+			get {
+				return allocationsCarryId;
+			}
+		}
+		public void AllocationsCarryIdReceived () {
+			allocationsCarryId = true;
+		}
+		
 		public DirectivesHandler () {
 			allocationsCarryCallerMethod = false;
+			allocationsHaveStackTrace = false;
+			allocationsCarryId = false;
 		}
 	}
 	
-	public class BaseProfilerEventHandler<LC,LM,UFR,UFI,MR,EH,HO,HS> : IProfilerEventHandler<LC,LM,UFR,UFI,MR,EH,HO,HS> where LC : ILoadedClass where LM : ILoadedMethod<LC> where UFR : IUnmanagedFunctionFromRegion where UFI : IUnmanagedFunctionFromID<MR,UFR> where MR : IExecutableMemoryRegion<UFR> where EH : ILoadedElementHandler<LC,LM,UFR,UFI,MR,HO,HS> where HO: IHeapObject<HO,LC> where HS: IHeapSnapshot<HO,LC> {
-		DirectivesHandler directives = new DirectivesHandler ();
+	public class BaseProfilerEventHandler<LC,LM,UFR,UFI,MR,EH,HO,HS> : IProfilerEventHandler<LC,LM,UFR,UFI,MR,EH,HO,HS> where LC : ILoadedClass where LM : ILoadedMethod<LC> where UFR : IUnmanagedFunctionFromRegion<UFR> where UFI : IUnmanagedFunctionFromID<MR,UFR> where MR : IExecutableMemoryRegion<UFR> where EH : ILoadedElementHandler<LC,LM,UFR,UFI,MR,HO,HS> where HO: IHeapObject<HO,LC> where HS: IHeapSnapshot<HO,LC> {
+		DirectivesHandler directives;
 		public DirectivesHandler Directives {
 			get {
 				return directives;
@@ -665,9 +722,22 @@ namespace Mono.Profiler {
 				return loadedElements;
 			}
 		}
+		public virtual Double TicksPerCounterUnit {
+			get {
+				return 0;
+			}
+		}
+		public virtual TimeSpan ClicksToTimeSpan (ulong clicks) {
+			return TimeSpan.FromTicks (0);
+		}
 		
 		public BaseProfilerEventHandler (EH loadedElements) {
 			this.loadedElements = loadedElements;
+			this.directives = new DirectivesHandler ();
+		}
+		protected BaseProfilerEventHandler (IProfilerEventHandler<LC,LM,UFR,UFI,MR,EH,HO,HS> baseHandler) {
+			this.loadedElements = baseHandler.LoadedElements;
+			this.directives = baseHandler.Directives;
 		}
 		
 		public virtual void Start (uint version, string runtimeFile, ProfilerFlags flags, ulong startCounter, DateTime startTime) {}
@@ -690,7 +760,7 @@ namespace Mono.Profiler {
 		public virtual void ClassStartUnload (LC c, ulong counter) {}
 		public virtual void ClassEndUnload (LC c, ulong counter) {}
 		
-		public virtual void Allocation (LC c, uint size, LM caller, bool jitTime, ulong counter) {}
+		public virtual void Allocation (LC c, uint size, LM caller, bool jitTime, ulong objectId, ulong counter) {}
 		public virtual void Exception (LC c, ulong counter) {}
 		
 		public virtual void MethodEnter (LM m, ulong counter) {}
@@ -699,6 +769,8 @@ namespace Mono.Profiler {
 		public virtual void MethodJitEnd (LM m, ulong counter, bool success) {}
 		public virtual void MethodFreed (LM m, ulong counter) {}
 		
+		public virtual void AdjustStack (uint lastValidFrame, uint topSectionSize, StackSectionElement<LC,LM>[] topSection) {}
+		
 		public virtual void MethodStatisticalHit (LM m) {}
 		public virtual void UnknownMethodStatisticalHit () {}
 		public virtual void UnmanagedFunctionStatisticalHit (UFR f) {}
@@ -730,9 +802,202 @@ namespace Mono.Profiler {
 		public virtual void AllocationSummaryStart (uint collection, ulong startCounter, DateTime startTime) {}
 		public virtual void ClassAllocationSummary (LC c, uint reachableInstances, uint reachableBytes, uint unreachableInstances, uint unreachableBytes) {}
 		public virtual void AllocationSummaryEnd (uint collection, ulong endCounter, DateTime endTime) {}
+		
+		public virtual void InitializeData (byte[] data, uint currentOffset) {}
+		public virtual void DataProcessed (uint offset) {}
+	}
+
+	public class DebugProfilerEventHandler<LC,LM,UFR,UFI,MR,EH,HO,HS> : BaseProfilerEventHandler<LC,LM,UFR,UFI,MR,EH,HO,HS> where LC : ILoadedClass where LM : ILoadedMethod<LC> where UFR : IUnmanagedFunctionFromRegion<UFR> where UFI : IUnmanagedFunctionFromID<MR,UFR> where MR : IExecutableMemoryRegion<UFR> where EH : ILoadedElementHandler<LC,LM,UFR,UFI,MR,HO,HS> where HO: IHeapObject<HO,LC> where HS: IHeapSnapshot<HO,LC> {
+		uint currentOffset;
+		public uint CurrentOffset {
+			get {
+				return currentOffset;
+			}
+		}
+		
+		byte[] data;
+		TextWriter output;
+		
+		public DebugProfilerEventHandler (IProfilerEventHandler<LC,LM,UFR,UFI,MR,EH,HO,HS> baseHandler, TextWriter output) : base (baseHandler) {
+			this.output = output;
+			this.data = null;
+			this.currentOffset = 0;
+		}
+		
+		public override void Start (uint version, string runtimeFile, ProfilerFlags flags, ulong startCounter, DateTime startTime) {
+			output.WriteLine ("");
+		}
+		public override void End (uint version, ulong endCounter, DateTime endTime) {
+			output.WriteLine ("");
+		}
+		
+		public override void StartBlock (ulong startCounter, DateTime startTime, ulong threadId) {
+			output.WriteLine ("StartBlock: startCounter {0}, startTime {1}, threadId {2}", startCounter, startTime, threadId);
+		}
+		public override void EndBlock (ulong endCounter, DateTime endTime, ulong threadId) {
+			output.WriteLine ("StartBlock: endCounter {0}, endTime {1}, threadId {2}", endCounter, endTime, threadId);
+		}
+		
+		public override void ModuleLoaded (ulong threadId, ulong startCounter, ulong endCounter, string name, bool success) {
+			output.WriteLine ("ModuleLoaded");
+		}
+		public override void ModuleUnloaded (ulong threadId, ulong startCounter, ulong endCounter, string name) {
+			output.WriteLine ("ModuleUnloaded");
+		}
+		public override void AssemblyLoaded (ulong threadId, ulong startCounter, ulong endCounter, string name, bool success) {
+			output.WriteLine ("AssemblyLoaded");
+		}
+		public override void AssemblyUnloaded (ulong threadId, ulong startCounter, ulong endCounter, string name) {
+			output.WriteLine ("AssemblyUnloaded");
+		}
+		public override void ApplicationDomainLoaded (ulong threadId, ulong startCounter, ulong endCounter, string name, bool success) {
+			output.WriteLine ("ApplicationDomainLoaded");
+		}
+		public override void ApplicationDomainUnloaded (ulong threadId, ulong startCounter, ulong endCounter, string name) {
+			output.WriteLine ("ApplicationDomainUnloaded");
+		}
+		
+		public override void SetCurrentThread (ulong threadId) {
+			output.WriteLine ("SetCurrentThread");
+		}
+		
+		public override void ClassStartLoad (LC c, ulong counter) {
+			output.WriteLine ("ClassStartLoad");
+		}
+		public override void ClassEndLoad (LC c, ulong counter, bool success) {
+			output.WriteLine ("ClassEndLoad");
+		}
+		public override void ClassStartUnload (LC c, ulong counter) {
+			output.WriteLine ("ClassStartUnload");
+		}
+		public override void ClassEndUnload (LC c, ulong counter) {
+			output.WriteLine ("ClassEndUnload");
+		}
+		
+		public override void Allocation (LC c, uint size, LM caller, bool jitTime, ulong objectId, ulong counter) {
+			output.WriteLine ("Allocation [classId {0}({1}), size {2}, callerId {3}), jitTime {4}, counter {5}]", c.ID, c.Name, size, caller != null ? caller.ID : 0, jitTime, counter);
+		}
+		public override void Exception (LC c, ulong counter) {
+			output.WriteLine ("Exception");
+		}
+		
+		public override void MethodEnter (LM m, ulong counter) {
+			output.WriteLine ("MethodEnter");
+		}
+		public override void MethodExit (LM m, ulong counter) {
+			output.WriteLine ("MethodExit");
+		}
+		public override void MethodJitStart (LM m, ulong counter) {
+			output.WriteLine ("MethodJitStart");
+		}
+		public override void MethodJitEnd (LM m, ulong counter, bool success) {
+			output.WriteLine ("MethodJitEnd");
+		}
+		public override void MethodFreed (LM m, ulong counter) {
+			output.WriteLine ("MethodFreed");
+		}
+		
+		public override void AdjustStack (uint lastValidFrame, uint topSectionSize, StackSectionElement<LC,LM>[] topSection) {
+			output.WriteLine ("AdjustStack");
+		}
+		
+		public override void MethodStatisticalHit (LM m) {
+			output.WriteLine ("MethodStatisticalHit");
+		}
+		public override void UnknownMethodStatisticalHit () {
+			output.WriteLine ("UnknownMethodStatisticalHit");
+		}
+		public override void UnmanagedFunctionStatisticalHit (UFR f) {
+			output.WriteLine ("UnmanagedFunctionStatisticalHit");
+		}
+		public override void UnmanagedFunctionStatisticalHit (UFI f) {
+			output.WriteLine ("UnmanagedFunctionStatisticalHit");
+		}
+		public override void UnknownUnmanagedFunctionStatisticalHit (MR region, uint offset) {
+			output.WriteLine ("UnknownUnmanagedFunctionStatisticalHit");
+		}
+		public override void UnknownUnmanagedFunctionStatisticalHit (ulong address) {
+			output.WriteLine ("UnknownUnmanagedFunctionStatisticalHit");
+		}
+		public override void StatisticalCallChainStart (uint chainDepth) {
+			output.WriteLine ("StatisticalCallChainStart");
+		}
+		
+		public override void ThreadStart (ulong threadId, ulong counter) {
+			output.WriteLine ("ThreadStart");
+		}
+		public override void ThreadEnd (ulong threadId, ulong counter) {
+			output.WriteLine ("ThreadEnd");
+		}
+		
+		public override void GarbageCollectionStart (uint collection, uint generation, ulong counter) {
+			output.WriteLine ("GarbageCollectionStart");
+		}
+		public override void GarbageCollectionEnd (uint collection, uint generation, ulong counter) {
+			output.WriteLine ("GarbageCollectionEnd");
+		}
+		public override void GarbageCollectionMarkStart (uint collection, uint generation, ulong counter) {
+			output.WriteLine ("GarbageCollectionMarkStart");
+		}
+		public override void GarbageCollectionMarkEnd (uint collection, uint generation, ulong counter) {
+			output.WriteLine ("GarbageCollectionMarkEnd");
+		}
+		public override void GarbageCollectionSweepStart (uint collection, uint generation, ulong counter) {
+			output.WriteLine ("GarbageCollectionSweepStart");
+		}
+		public override void GarbageCollectionSweepEnd (uint collection, uint generation, ulong counter) {
+			output.WriteLine ("GarbageCollectionSweepEnd");
+		}
+		public override void GarbageCollectionResize (uint collection, ulong newSize) {
+			output.WriteLine ("GarbageCollectionResize");
+		}
+		public override void GarbageCollectionStopWorldStart (uint collection, uint generation, ulong counter) {
+			output.WriteLine ("GarbageCollectionStopWorldStart");
+		}
+		public override void GarbageCollectionStopWorldEnd (uint collection, uint generation, ulong counter) {
+			output.WriteLine ("GarbageCollectionStopWorldEnd");
+		}
+		public override void GarbageCollectionStartWorldStart (uint collection, uint generation, ulong counter) {
+			output.WriteLine ("GarbageCollectionStartWorldStart");
+		}
+		public override void GarbageCollectionStartWorldEnd (uint collection, uint generation, ulong counter) {
+			output.WriteLine ("GarbageCollectionStartWorldEnd");
+		}
+		
+		public override void HeapReportStart (HS snapshot) {
+			output.WriteLine ("HeapReportStart");
+		}
+		public override void HeapObjectUnreachable (LC c, uint size) {
+			output.WriteLine ("HeapObjectUnreachable");
+		}
+		public override void HeapObjectReachable (HO o) {
+			output.WriteLine ("HeapObjectReachable");
+		}
+		public override void HeapReportEnd (HS snapshot) {
+			output.WriteLine ("HeapReportEnd");
+		}
+		
+		public override void AllocationSummaryStart (uint collection, ulong startCounter, DateTime startTime) {
+			output.WriteLine ("AllocationSummaryStart");
+		}
+		public override void ClassAllocationSummary (LC c, uint reachableInstances, uint reachableBytes, uint unreachableInstances, uint unreachableBytes) {
+			output.WriteLine ("ClassAllocationSummary");
+		}
+		public override void AllocationSummaryEnd (uint collection, ulong endCounter, DateTime endTime) {
+			output.WriteLine ("AllocationSummaryEnd");
+		}
+		
+		public override void InitializeData (byte[] data, uint currentOffset) {
+			this.data = data;
+			this.currentOffset = currentOffset;
+		}
+		public override void DataProcessed (uint offset) {
+			BlockData.DumpData (data, output, currentOffset, offset);
+			currentOffset = offset;
+		}
 	}
 	
-	public class BaseExecutableMemoryRegion<UFR> : BaseLoadedElement, IExecutableMemoryRegion<UFR> where UFR : IUnmanagedFunctionFromRegion, new() {
+	public class BaseExecutableMemoryRegion<UFR> : BaseLoadedElement, IExecutableMemoryRegion<UFR> where UFR : IUnmanagedFunctionFromRegion<UFR>, new() {
 		uint fileOffset;
 		public uint FileOffset {
 			get {
@@ -761,6 +1026,7 @@ namespace Mono.Profiler {
 			result.Name = name;
 			result.StartOffset = offset;
 			result.EndOffset = offset;
+			result.Region = this;
 			functions.Add (result);
 			return result;
 		}
@@ -833,7 +1099,7 @@ namespace Mono.Profiler {
 		}
 	}
 	
-	public class LoadedElementHandler<LC,LM,UFR,UFI,MR,HO,HS> : ILoadedElementHandler<LC,LM,UFR,UFI,MR,HO,HS> where LC : ILoadedClass where LM : ILoadedMethod<LC> where UFR : IUnmanagedFunctionFromRegion where UFI : IUnmanagedFunctionFromID<MR,UFR> where MR : IExecutableMemoryRegion<UFR> where HO: IHeapObject<HO,LC> where HS: IHeapSnapshot<HO,LC> {
+	public class LoadedElementHandler<LC,LM,UFR,UFI,MR,HO,HS> : ILoadedElementHandler<LC,LM,UFR,UFI,MR,HO,HS> where LC : ILoadedClass where LM : ILoadedMethod<LC> where UFR : IUnmanagedFunctionFromRegion<UFR> where UFI : IUnmanagedFunctionFromID<MR,UFR> where MR : IExecutableMemoryRegion<UFR> where HO: IHeapObject<HO,LC> where HS: IHeapSnapshot<HO,LC> {
 		ILoadedElementFactory<LC,LM,UFR,UFI,MR,HO,HS> factory;
 		
 		int loadedClassesCount;
@@ -966,8 +1232,8 @@ namespace Mono.Profiler {
 		}
 		
 		List<HS> heapSnapshots;
-		public HS NewHeapSnapshot (uint collection, ulong startCounter, DateTime startTime, ulong endCounter, DateTime endTime, LC[] initialAllocations, bool recordSnapshot) {
-			HS result = factory.NewHeapSnapshot (collection, startCounter, startTime, endCounter, endTime, initialAllocations, recordSnapshot);
+		public HS NewHeapSnapshot (uint collection, ulong startCounter, DateTime startTime, ulong endCounter, DateTime endTime, TimeSpan headerStartTime, LC[] initialAllocations, bool recordSnapshot) {
+			HS result = factory.NewHeapSnapshot (collection, startCounter, startTime, endCounter, endTime, headerStartTime, initialAllocations, recordSnapshot);
 			heapSnapshots.Add (result);
 			return result;
 		}
