@@ -40,8 +40,44 @@ namespace Gendarme.Rules.Correctness {
 	// the correct way of handling this back then). The goal is to make this ctor 
 	// non-visible so people can't instantiate it (e.g. by mistake)
 
+	/// <summary>
+	/// This rule check for types that contains only static members and warn if a visible
+	/// instance constructor is present in it. This was a common mistake in the framework
+	/// 1.x since C# adds a default, public, constructor when none is provided. Code using
+	/// the framework 2.0 (and later) should change this type, if possible, into a static
+	/// type.
+	/// </summary>
+	/// <example>
+	/// Bad example:
+	/// <code>
+	/// // it is possible to instantiate this class since the 
+	/// // C# compiler adds a default, public, constructor
+	/// public class Class {
+	///	public static void Method ()
+	///	{
+	///	}
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example:
+	/// <code>
+	/// public class Class {
+	///	// this class cannot be instantiated
+	///	private Class ()
+	///	{
+	///	}
+	///	
+	///	public static void Method ()
+	///	{
+	///	}
+	/// }
+	/// </code>
+	/// </example>
+
 	[Problem ("This type (and ancestors) contains only static fields and methods but has a visible, non static, constructor.")]
 	[Solution ("You should remove the visible constructor to make sure this type cannot be instancied, or if using 2.0+, change it to a static type.")]
+	[FxCopCompatibility ("Microsoft.Design", "CA1053:StaticHolderTypesShouldNotHaveConstructors")]
 	public class AvoidConstructorsInStaticTypesRule : Rule, ITypeRule {
 
 		static bool IsAllStatic (TypeDefinition type)
@@ -49,20 +85,26 @@ namespace Gendarme.Rules.Correctness {
 			if (type == null)
 				return false;
 
-			foreach (MethodDefinition ctor in type.Constructors) {
-				// let's the default ctor pass (since it's always here for 1.x code)
-				if (!ctor.IsStatic && (ctor.Parameters.Count > 1))
-					return false;
+			if (type.HasConstructors) {
+				foreach (MethodDefinition ctor in type.Constructors) {
+					// let's the default ctor pass (since it's always here for 1.x code)
+					if (!ctor.IsStatic && ctor.HasParameters)
+						return false;
+				}
 			}
 
-			foreach (MethodDefinition method in type.Methods) {
-				if (!method.IsStatic)
-					return false;
+			if (type.HasMethods) {
+				foreach (MethodDefinition method in type.Methods) {
+					if (!method.IsStatic)
+						return false;
+				}
 			}
 
-			foreach (FieldDefinition field in type.Fields) {
-				if (!field.IsStatic)
-					return false;
+			if (type.HasFields) {
+				foreach (FieldDefinition field in type.Fields) {
+					if (!field.IsStatic)
+						return false;
+				}
 			}
 
 			if (type.BaseType.FullName == "System.Object")
