@@ -36,8 +36,79 @@ using Gendarme.Framework.Rocks;
 
 namespace Gendarme.Rules.Design {
 
+	/// <summary>
+	/// The rule ensure that all overloaded operators are also accessible used named 
+	/// alternatives because some languages, like VB.NET, cannot use overloaded operators. 
+	/// For those languages named methods should be implemented that provide the same
+	/// functionality. This rule checks for a named alternative for each overloaded operator.
+	/// <list type="bullet">
+	/// <item><term>op_UnaryPlus</term><description>Plus</description></item>
+	/// <item><term>op_UnaryNegation</term><description>Negate</description></item>
+	/// <item><term>op_LogicalNot</term><description>LogicalNot</description></item>
+	/// <item><term>op_OnesComplement</term><description>OnesComplement</description></item>
+	/// </list>
+	/// <list type="bullet">
+	/// <item><term>op_Increment</term><description>Increment</description></item>
+	/// <item><term>op_Decrement</term><description>Decrement</description></item>
+	/// <item><term>op_True</term><description>IsTrue</description></item>
+	/// <item><term>op_False</term><description>IsFalse</description></item>
+	/// </list>
+	/// <list type="bullet">
+	/// <item><term>op_Addition</term><description>Add</description></item>
+	/// <item><term>op_Subtraction</term><description>Subtract</description></item>
+	/// <item><term>op_Multiply</term><description>Multiply</description></item>
+	/// <item><term>op_Division</term><description>Divide</description></item>
+	/// <item><term>op_Modulus</term><description>Modulus</description></item>
+	/// </list>
+	/// <list type="bullet">
+	/// <item><term>op_BitwiseAnd</term><description>BitwiseAnd</description></item>
+	/// <item><term>op_BitwiseOr</term><description>BitwiseOr</description></item>
+	/// <item><term>op_ExclusiveOr</term><description>ExclusiveOr</description></item>
+	/// </list>
+	/// <list type="bullet">
+	/// <item><term>op_LeftShift</term><description>LeftShift</description></item>
+	/// <item><term>op_RightShift</term><description>RightShift</description></item>
+	/// </list>
+	/// <list type="bullet">
+	/// <item><term>op_Equality</term><description>Equals</description></item>
+	/// <item><term>op_Inequality</term><description>(not) Equals</description></item>
+	/// <item><term>op_GreaterThan</term><description>Compare</description></item>
+	/// <item><term>op_LessThan</term><description>Compare</description></item>
+	/// <item><term>op_GreaterThanOrEqual</term><description>Compare</description></item>
+	/// <item><term>op_LessThanOrEqual</term><description>Compare</description></item>
+	/// </list>
+	/// </summary>
+	/// <example>
+	/// Bad example:
+	/// <code>
+	/// class DoesNotImplementAlternative {
+	///	public static int operator + (DoesNotOverloadOperatorEquals a, DoesNotOverloadOperatorEquals b)
+	///	{
+	///		return 0;
+	///	}
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example:
+	/// <code>
+	/// class DoesImplementAdd {
+	///	public static int operator + (DoesImplementAdd a, DoesImplementAdd b)
+	///	{
+	///		return 0;
+	///	}
+	///	
+	///	public int Add (DoesImplementAdd a)
+	///	{
+	///		return this + a;
+	///	}
+	///}
+	/// </code>
+	/// </example>
+
 	[Problem ("This type contains overloads for some operators but doesn't provide named alternatives.")]
 	[Solution ("Add named methods equivalent to the operators for language that do not support them (e.g. VB.NET).")]
+	[FxCopCompatibility ("Microsoft.Usage", "CA2225:OperatorOverloadsHaveNamedAlternates")]
 	public class ProvideAlternativeNamesForOperatorOverloadsRule : Rule, ITypeRule {
 
 		static string [] NoParameter = new string [] { };
@@ -82,7 +153,7 @@ namespace Gendarme.Rules.Design {
 
 		public RuleResult CheckType (TypeDefinition type)
 		{
-			if (type.IsEnum || type.IsInterface)
+			if (type.IsEnum || type.IsInterface || type.IsDelegate ())
 				return RuleResult.DoesNotApply;
 
 			foreach (var kv in AlternativeMethodNames) {
