@@ -33,7 +33,10 @@ using System.Net;
 
 using Mono.Cecil;
 using Mono.Cecil.Cil;
+
 using Gendarme.Framework;
+using Gendarme.Framework.Engines;
+using Gendarme.Framework.Helpers;
 
 using ICSharpCode.SharpZipLib.Zip;
 
@@ -41,23 +44,42 @@ using MoMA.Analyzer.MoMAWebService;
 
 namespace Gendarme.Rules.Portability {
 
+	/// <summary>
+	/// This rule uses MoMA definition files to analyze assemblies and warns if called methods are:
+	/// <list type="bullet">
+	/// <item>
+	/// <description>marked with a <c>[MonoTODO]</c> attribute;</description>
+	/// </item>
+	/// <item>
+	/// <description>throw a <c>NotImplementedException</c>; or</description>
+	/// </item>
+	/// <item>
+	/// <description>do not exist in the current version of Mono::</description>
+	/// </item>
+	/// </list>
+	/// The rule looks for the definitions in <c>~/.local/share/Gendarme/definitions.zip</c>. 
+	/// If the file is missing it will try to download the latest version.
+	/// </summary>
+	/// <remarks>This rule does not replace MoMA (which is obviously the easiest solution for Windows developers). It's goal is to help analyze multiple portability issues from the Linux side (where Gendarme is most likely being used)</remarks>
+
 	[Problem ("The method has some known limitations when used with the Mono:: runtime.")]
 	[Solution ("Check if this code is critical to your application. Also make sure your definition files are up to date.")]
+	[EngineDependency (typeof (OpCodeEngine))]
 	public class MonoCompatibilityReviewRule : Rule, IMethodRule {
 
-		private const string NotImplementedMessage = "{0} is not implemented.{1}";
-		private const string MissingMessage = "{0} is missing from Mono.{1}";
+		private const string NotImplementedMessage = "{0} is not implemented.";
+		private const string MissingMessage = "{0} is missing from Mono.";
 		private const string TodoMessage = "{0} is marked with a [MonoTODO] attribute: {1}.";
 
-		private Dictionary<string, string> NotImplementedInternal; //value is unused
-		private Dictionary<string, string> MissingInternal; //value is unused
+		private HashSet<string> NotImplementedInternal; //value is unused
+		private HashSet<string> MissingInternal; //value is unused
 		private Dictionary<string, string> TodoInternal; //value = TODO Description
 
-		public Dictionary<string, string> NotImplemented {
+		public HashSet<string> NotImplemented {
 			get { return NotImplementedInternal; }
 		}
 
-		public Dictionary<string, string> Missing {
+		public HashSet<string> Missing {
 			get { return MissingInternal; }
 		}
 
@@ -89,13 +111,13 @@ namespace Gendarme.Rules.Portability {
 					while ((ze = zs.GetNextEntry ()) != null) {
 						switch (ze.Name) {
 						case "exception.txt":
-							NotImplementedInternal = Read (new StreamReader (zs), false);
+							NotImplementedInternal = Read (new StreamReader (zs));
 							break;
 						case "missing.txt":
-								MissingInternal = Read (new StreamReader (zs), false);
+							MissingInternal = Read (new StreamReader (zs));
 							break;
 						case "monotodo.txt":
-							TodoInternal = Read (new StreamReader (zs), true);
+							TodoInternal = ReadWithComments (new StreamReader (zs));
 							break;
 						default:
 							break;
@@ -129,31 +151,35 @@ namespace Gendarme.Rules.Portability {
 			return true;
 		}
 
-		private static Dictionary<string, string> Read (StreamReader reader, bool split)
+		private static Dictionary<string, string> ReadWithComments (TextReader reader)
 		{
 			Dictionary<string, string> dict = new Dictionary<string, string> ();
 			string line;
 			while ((line = reader.ReadLine ()) != null) {
-				if (split) {
-					string [] parts = line.Split ('-');
-					dict.Add (parts [0], parts [1]);
+				int split = line.IndexOf ('-');
+				string target = line.Substring (0, split);
+				// are there comments ? (many entries don't have any)
+				if (split == line.Length - 1) {
+					dict.Add (target, null);
 				} else {
-					dict.Add (line, null);
+					dict.Add (target, line.Substring (split + 1));
 				}
 			}
 			return dict;
 		}
 
-		private void Check (Dictionary<string, string> dict, MethodDefinition method, Instruction ins, string error, Severity severity)
+		private static HashSet<string> Read (TextReader reader)
 		{
-			string callee = ins.Operand.ToString ();
-			if (!dict.ContainsKey (callee))
-				return;
-
-			string message = String.Format (error, callee, dict [callee]);
-			// confidence is Normal since we can't be sure if MoMA data is up to date
-			Runner.Report (method, ins, severity, Confidence.Normal, message);
+			HashSet<string> set = new HashSet<string> ();
+			string line;
+			while ((line = reader.ReadLine ()) != null) {
+				set.Add (line);
+			}
+			return set;
 		}
+
+		// this correspond to Call, Calli, Callvirt, Newobj, Initobj, Ldftn, Ldvirtftn
+		private static OpCodeBitmask mask = new OpCodeBitmask (0x18000000000, 0x4400000000000, 0x0, 0x40060);
 		
 		public RuleResult CheckMethod (MethodDefinition method)
 		{
@@ -161,34 +187,40 @@ namespace Gendarme.Rules.Portability {
 			if (!method.HasBody)
 				return RuleResult.DoesNotApply;
 
+			// check if any instruction refers to methods or types that MoMA could track
+			if (!mask.Intersect (OpCodeEngine.GetBitmask (method)))
+				return RuleResult.DoesNotApply;
+
 			// rule applies
 
 			foreach (Instruction ins in method.Body.Instructions) {
-				switch (ins.OpCode.Code) {
-				case Code.Call:
-				case Code.Calli:
-				case Code.Callvirt:
-				case Code.Newobj:
-				case Code.Initobj:
-				case Code.Ldftn:
-				case Code.Ldvirtftn:
-					// calling not implemented method is very likely not to work == High
-					if (NotImplemented != null) {
-						Check (NotImplemented, method, ins, NotImplementedMessage, Severity.High);
-					}
+				// look for any instruction that could use something incomplete
+				if (!mask.Get (ins.OpCode.Code))
+					continue;
 
-					// calling missing methods can't work == Critical
-					if (Missing != null) {
-						Check (Missing, method, ins, MissingMessage, Severity.Critical);
-					}
+				// this (MethodReference.ToString) is costly so we do it once for the three checks
+				string callee = ins.Operand.ToString ();
 
-					// calling todo methods migh work with some limitations == Medium
-					if (ToDo != null) {
-						Check (ToDo, method, ins, TodoMessage, Severity.Medium);
+				// calling not implemented method is very likely not to work == High
+				if ((NotImplemented != null) && NotImplementedInternal.Contains (callee)) {
+					string message = String.Format (NotImplementedMessage, callee);
+					// confidence is Normal since we can't be sure if MoMA data is up to date
+					Runner.Report (method, ins, Severity.High, Confidence.Normal, message);
+				}
+
+				// calling missing methods can't work == Critical
+				if ((Missing != null) && Missing.Contains (callee)) {
+					string message = String.Format (MissingMessage, callee);
+					Runner.Report (method, ins, Severity.Critical, Confidence.Normal, message);
+				}
+
+				// calling todo methods migh work with some limitations == Medium
+				if (ToDo != null) {
+					string value;
+					if (ToDo.TryGetValue (callee, out value)) {
+						string message = String.Format (TodoMessage, callee, value);
+						Runner.Report (method, ins,  Severity.Medium, Confidence.Normal, message);
 					}
-					break;
-				default:
-					break;
 				}
 			}
 
@@ -196,3 +228,4 @@ namespace Gendarme.Rules.Portability {
 		}
 	}
 }
+
