@@ -33,14 +33,46 @@ using Gendarme.Framework.Rocks;
 
 namespace Gendarme.Rules.Security {
 
+	/// <summary>
+	/// This rule checks if a class exposes native fields. Native fields should not
+	/// be public because you lose control over their lifetime (other code could free
+	/// the memory or use it after it has been freed).
+	/// </summary>
+	/// <example>
+	/// Bad example:
+	/// <code>
+	/// public class HasPublicNativeField {
+	///	public IntPtr NativeField;
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example (hide):
+	/// <code>
+	/// class HasPrivateNativeField {
+	///	private IntPtr NativeField;
+	///	public void DoSomethingWithNativeField ();
+	/// }
+	/// </code>
+	/// </example>
+	/// <example>
+	/// Good example (read-only):
+	/// <code>
+	/// class HasReadOnlyNativeField {
+	///	public readonly IntPtr NativeField;
+	/// }
+	/// </code>
+	/// </example>
+
 	[Problem ("This type expose native fields that aren't read-only.")]
 	[Solution ("Native fields are best hidden or, if required, read-only.")]
+	[FxCopCompatibility ("Microsoft.Security", "CA2111:PointersShouldNotBeVisible")]
 	public class NativeFieldsShouldNotBeVisibleRule : Rule, ITypeRule {
 
 		public RuleResult CheckType (TypeDefinition type)
 		{
-			// rule does not apply to interface, enumerations or delegates
-			if (type.IsInterface || type.IsEnum || type.IsDelegate ())
+			// rule does not apply to interface, enumerations and delegates or to types without fields
+			if (type.IsInterface || type.IsEnum || !type.HasFields || type.IsDelegate ())
 				return RuleResult.DoesNotApply;
 
 			foreach (FieldDefinition field in type.Fields) {
