@@ -30,13 +30,17 @@ using System;
 using System.Collections;
 using System.IO;
 using System.Reflection;
+using System.Windows.Forms;
 
 using Mono.Cecil;
-using NUnit.Framework;
 using Gendarme.Framework;
+using Gendarme.Framework.Rocks;
 using Gendarme.Rules.Smells;
 
-using System.Windows.Forms;
+using NUnit.Framework;
+using Test.Rules.Definitions;
+using Test.Rules.Fixtures;
+using Test.Rules.Helpers;
 
 //Stubs for the Gtk testing.
 namespace Gtk {
@@ -51,6 +55,386 @@ namespace Gtk {
 }
 
 namespace Test.Rules.Smells {
+	public class LongStaticConstructorWithFields {
+		static readonly int foo;
+		static string bar;
+		static object baz;
+
+		static LongStaticConstructorWithFields () {
+			foo = 5;
+			bar = "MyString";
+			baz = new object ();
+			Console.WriteLine ("I'm writting a test, and I will fill a screen with some useless code");
+			IList list = new ArrayList ();
+			list.Add ("Foo");
+			list.Add (4);
+
+			IEnumerator listEnumerator = list.GetEnumerator ();
+			while (listEnumerator.MoveNext ())
+				Console.WriteLine (listEnumerator.Current);
+
+			try {
+				list.Add ("Bar");
+				list.Add ('a');
+			}
+			catch (NotSupportedException exception) {
+				Console.WriteLine (exception.Message);
+				Console.WriteLine (exception);
+			}
+
+			foreach (object value in list) {
+				Console.Write (value);
+				Console.Write (Environment.NewLine);
+			}
+			
+			int x = 0;
+
+			for (int i = 0; i < 100; i++)
+				x++;
+			Console.WriteLine (x);
+	
+			string useless = "Useless String";
+
+			if (useless.Equals ("Other useless")) {
+				useless = String.Empty;
+				Console.WriteLine ("Other useless string");
+			}
+			
+			useless = String.Concat (useless," 1");
+			
+			for (int j = 0; j < useless.Length; j++) {
+				if (useless[j] == 'u')
+					Console.WriteLine ("I have detected an u char");
+				else
+					Console.WriteLine ("I have detected an useless char");
+			}
+			
+			try {
+				foreach (string environmentVariable in Environment.GetEnvironmentVariables ().Keys)
+					Console.WriteLine (environmentVariable);
+			}
+			catch (System.Security.SecurityException exception) {
+				Console.WriteLine (exception.Message);
+				Console.WriteLine (exception);
+			}
+
+			Console.WriteLine ("I will add more useless code !!");
+			
+			try {
+				if (!(File.Exists ("foo.txt"))) {
+					File.Create ("foo.txt");	
+					File.Delete ("foo.txt");
+				}
+			}
+			catch (IOException exception) {
+				Console.WriteLine (exception.Message);
+				Console.WriteLine (exception);
+			}
+		}
+	}
+
+	public class LongStaticConstructorWithoutFields {
+		static LongStaticConstructorWithoutFields () {
+			Console.WriteLine ("I'm writting a test, and I will fill a screen with some useless code");
+			IList list = new ArrayList ();
+			list.Add ("Foo");
+			list.Add (4);
+			list.Add (6);
+
+			IEnumerator listEnumerator = list.GetEnumerator ();
+			while (listEnumerator.MoveNext ())
+				Console.WriteLine (listEnumerator.Current);
+
+			try {
+				list.Add ("Bar");
+				list.Add ('a');
+			}
+			catch (NotSupportedException exception) {
+				Console.WriteLine (exception.Message);
+				Console.WriteLine (exception);
+			}
+
+			foreach (object value in list) {
+				Console.Write (value);
+				Console.Write (Environment.NewLine);
+			}
+			
+			int x = 0;
+
+			for (int i = 0; i < 100; i++)
+				x++;
+			Console.WriteLine (x);
+	
+			string useless = "Useless String";
+
+			if (useless.Equals ("Other useless")) {
+				useless = String.Empty;
+				Console.WriteLine ("Other useless string");
+			}
+			
+			useless = String.Concat (useless," 1");
+			
+			for (int j = 0; j < useless.Length; j++) {
+				if (useless[j] == 'u')
+					Console.WriteLine ("I have detected an u char");
+				else
+					Console.WriteLine ("I have detected an useless char");
+			}
+			
+			try {
+				foreach (string environmentVariable in Environment.GetEnvironmentVariables ().Keys)
+					Console.WriteLine (environmentVariable);
+			}
+			catch (System.Security.SecurityException exception) {
+				Console.WriteLine (exception.Message);
+				Console.WriteLine (exception);
+			}
+
+			Console.WriteLine ("I will add more useless code !!");
+			
+			try {
+				if (!(File.Exists ("foo.txt"))) {
+					File.Create ("foo.txt");	
+					File.Delete ("foo.txt");
+				}
+			}
+			catch (IOException exception) {
+				Console.WriteLine (exception.Message);
+				Console.WriteLine (exception);
+			}
+		}
+	}
+	
+	public class LongConstructorWithReadonlyFields {
+		readonly int foo;
+		readonly string bar, bar1, bar2, bar3;
+		readonly object baz, baz1, baz2, baz3;
+
+		public LongConstructorWithReadonlyFields () {
+			foo = 5;
+			bar = "MyString";
+			baz = new object ();
+			Console.WriteLine ("I'm writting a test, and I will fill a screen with some useless code");
+			IList list = new ArrayList ();
+			list.Add ("Foo");
+			list.Add (4);
+			list.Add (6);
+
+			IEnumerator listEnumerator = list.GetEnumerator ();
+			while (listEnumerator.MoveNext ())
+				Console.WriteLine (listEnumerator.Current);
+
+			try {
+				list.Add ("Bar");
+				list.Add ('a');
+			}
+			catch (NotSupportedException exception) {
+				Console.WriteLine (exception.Message);
+				Console.WriteLine (exception);
+			}
+
+			foreach (object value in list) {
+				Console.Write (value);
+				Console.Write (Environment.NewLine);
+			}
+			
+			int x = 0;
+
+			for (int i = 0; i < 100; i++)
+				x++;
+			Console.WriteLine (x);
+	
+			string useless = "Useless String";
+
+			if (useless.Equals ("Other useless")) {
+				useless = String.Empty;
+				Console.WriteLine ("Other useless string");
+			}
+			
+			useless = String.Concat (useless," 1");
+			
+			for (int j = 0; j < useless.Length; j++) {
+				if (useless[j] == 'u')
+					Console.WriteLine ("I have detected an u char");
+				else
+					Console.WriteLine ("I have detected an useless char");
+			}
+			
+			try {
+				foreach (string environmentVariable in Environment.GetEnvironmentVariables ().Keys)
+					Console.WriteLine (environmentVariable);
+			}
+			catch (System.Security.SecurityException exception) {
+				Console.WriteLine (exception.Message);
+				Console.WriteLine (exception);
+			}
+
+			Console.WriteLine ("I will add more useless code !!");
+			
+			try {
+				if (!(File.Exists ("foo.txt"))) {
+					File.Create ("foo.txt");	
+					File.Delete ("foo.txt");
+				}
+			}
+			catch (IOException exception) {
+				Console.WriteLine (exception.Message);
+				Console.WriteLine (exception);
+			}
+		}
+	}
+
+	public class LongConstructorWithFields {
+		readonly int foo;
+		string bar, bar1, bar2, bar3;
+		object baz, baz1, baz2, baz3;
+
+		public LongConstructorWithFields () {
+			foo = 5;
+			bar = "MyString";
+			baz = new object ();
+			Console.WriteLine ("I'm writting a test, and I will fill a screen with some useless code");
+			IList list = new ArrayList ();
+			list.Add ("Foo");
+			list.Add (4);
+			list.Add (6);
+
+			IEnumerator listEnumerator = list.GetEnumerator ();
+			while (listEnumerator.MoveNext ())
+				Console.WriteLine (listEnumerator.Current);
+
+			try {
+				list.Add ("Bar");
+				list.Add ('a');
+			}
+			catch (NotSupportedException exception) {
+				Console.WriteLine (exception.Message);
+				Console.WriteLine (exception);
+			}
+
+			foreach (object value in list) {
+				Console.Write (value);
+				Console.Write (Environment.NewLine);
+			}
+			
+			int x = 0;
+
+			for (int i = 0; i < 100; i++)
+				x++;
+			Console.WriteLine (x);
+	
+			string useless = "Useless String";
+
+			if (useless.Equals ("Other useless")) {
+				useless = String.Empty;
+				Console.WriteLine ("Other useless string");
+			}
+			
+			useless = String.Concat (useless," 1");
+			
+			for (int j = 0; j < useless.Length; j++) {
+				if (useless[j] == 'u')
+					Console.WriteLine ("I have detected an u char");
+				else
+					Console.WriteLine ("I have detected an useless char");
+			}
+			
+			try {
+				foreach (string environmentVariable in Environment.GetEnvironmentVariables ().Keys)
+					Console.WriteLine (environmentVariable);
+			}
+			catch (System.Security.SecurityException exception) {
+				Console.WriteLine (exception.Message);
+				Console.WriteLine (exception);
+			}
+
+			Console.WriteLine ("I will add more useless code !!");
+			
+			try {
+				if (!(File.Exists ("foo.txt"))) {
+					File.Create ("foo.txt");	
+					File.Delete ("foo.txt");
+				}
+			}
+			catch (IOException exception) {
+				Console.WriteLine (exception.Message);
+				Console.WriteLine (exception);
+			}
+		}
+	}
+
+	public class LongConstructorWithoutFields {
+		public LongConstructorWithoutFields () {
+			Console.WriteLine ("I'm writting a test, and I will fill a screen with some useless code");
+			IList list = new ArrayList ();
+			list.Add ("Foo");
+			list.Add (4);
+			list.Add (6);
+
+			IEnumerator listEnumerator = list.GetEnumerator ();
+			while (listEnumerator.MoveNext ())
+				Console.WriteLine (listEnumerator.Current);
+
+			try {
+				list.Add ("Bar");
+				list.Add ('a');
+			}
+			catch (NotSupportedException exception) {
+				Console.WriteLine (exception.Message);
+				Console.WriteLine (exception);
+			}
+
+			foreach (object value in list) {
+				Console.Write (value);
+				Console.Write (Environment.NewLine);
+			}
+			
+			int x = 0;
+
+			for (int i = 0; i < 100; i++)
+				x++;
+			Console.WriteLine (x);
+	
+			string useless = "Useless String";
+
+			if (useless.Equals ("Other useless")) {
+				useless = String.Empty;
+				Console.WriteLine ("Other useless string");
+			}
+			
+			useless = String.Concat (useless," 1");
+			
+			for (int j = 0; j < useless.Length; j++) {
+				if (useless[j] == 'u')
+					Console.WriteLine ("I have detected an u char");
+				else
+					Console.WriteLine ("I have detected an useless char");
+			}
+			
+			try {
+				foreach (string environmentVariable in Environment.GetEnvironmentVariables ().Keys)
+					Console.WriteLine (environmentVariable);
+			}
+			catch (System.Security.SecurityException exception) {
+				Console.WriteLine (exception.Message);
+				Console.WriteLine (exception);
+			}
+
+			Console.WriteLine ("I will add more useless code !!");
+			
+			try {
+				if (!(File.Exists ("foo.txt"))) {
+					File.Create ("foo.txt");	
+					File.Delete ("foo.txt");
+				}
+			}
+			catch (IOException exception) {
+				Console.WriteLine (exception.Message);
+				Console.WriteLine (exception);
+			}
+		}
+	}
+
 
 	public class MainWidget : Gtk.Bin {
 		protected virtual void Build () 
@@ -630,21 +1014,7 @@ namespace Test.Rules.Smells {
 		}
 	}
 
-	[TestFixture]
-	public class AvoidLongMethodsTest {
-		private IMethodRule rule;
-		private AssemblyDefinition assembly;
-		private MethodDefinition method;
-		private MessageCollection messageCollection;
-
-		[TestFixtureSetUp]
-		public void FixtureSetUp () 
-		{
-			string unit = Assembly.GetExecutingAssembly ().Location;
-			assembly = AssemblyFactory.GetAssembly (unit);
-			rule = new AvoidLongMethodsRule ();
-			messageCollection = null;
-		}
+	public class AvoidLongMethodsTest : MethodRuleTestFixture<AvoidLongMethodsRule> {
 
 		public void LongMethod () 
 		{
@@ -717,10 +1087,6 @@ namespace Test.Rules.Smells {
 			}
 		}
 
-		public void EmptyMethod () 
-		{
-		}
-
 		public void ShortMethod ()
 		{
 			try {
@@ -875,133 +1241,138 @@ namespace Test.Rules.Smells {
 			}
 		}
 
-		private MethodDefinition GetMethodForTest (string methodName) 
-		{
-			return GetMethodForTestFrom ("Test.Rules.Smells.AvoidLongMethodsTest", methodName);
-		}
-
-		private MethodDefinition GetMethodForTestFrom (string fullTypeName, string methodName) 
-		{
-			TypeDefinition type =  assembly.MainModule.Types[fullTypeName];
-			if (type != null) {
-				foreach (MethodDefinition method in type.Methods) {
-				if (method.Name == methodName)
-					return method;
-				}
-			}
-			return null;
-		}
-
 		[Test]
 		public void LongMethodTest () 
 		{
-			method = GetMethodForTest ("LongMethod");
-			messageCollection = rule.CheckMethod (method, new MinimalRunner ());
-			Assert.IsNotNull (messageCollection);
-			Assert.AreEqual (1, messageCollection.Count);
+			AssertRuleFailure<AvoidLongMethodsTest> ("LongMethod", 1);
 		}
 
 		[Test]
 		public void EmptyMethodTest () 
 		{
-			method = GetMethodForTest ("EmptyMethod");
-			messageCollection = rule.CheckMethod (method, new MinimalRunner ());
-			Assert.IsNull (messageCollection);
+			AssertRuleSuccess (SimpleMethods.EmptyMethod);
 		}
 
 		[Test]
 		public void ShortMethodTest () 
 		{
-			method = GetMethodForTest ("ShortMethod");
-			messageCollection = rule.CheckMethod (method, new MinimalRunner ());
-			Assert.IsNull (messageCollection);
+			AssertRuleSuccess <AvoidLongMethodsTest> ("ShortMethod");
 		}
 
 		[Test]
 		public void FalseBuildMethodTest ()
 		{
-			method = GetMethodForTest ("Build");
-			messageCollection = rule.CheckMethod (method, new MinimalRunner ());
-			Assert.IsNotNull (messageCollection);
-			Assert.AreEqual (1, messageCollection.Count);
+			AssertRuleFailure<AvoidLongMethodsTest> ("Build", 1);
 		}
-		
+
 		[Test]
 		public void WidgetBuildMethodTest () 
 		{
-			method = GetMethodForTestFrom ("Test.Rules.Smells.MainWidget", "Build");
-			messageCollection = rule.CheckMethod (method, new MinimalRunner ());
-			Assert.IsNull (messageCollection);
+			AssertRuleDoesNotApply<MainWidget> ("Build");
 		}
 
 		[Test]
 		public void WidgetInitializeComponentMethodTest () 
 		{
-			method = GetMethodForTestFrom ("Test.Rules.Smells.MainWidget", "InitializeComponent");
-			messageCollection = rule.CheckMethod (method, new MinimalRunner ());
-			Assert.IsNotNull (messageCollection);
-			Assert.AreEqual (1, messageCollection.Count);
+			AssertRuleFailure<MainWidget> ("InitializeComponent", 1);
 		}
 
-
 		[Test]
 		public void DialogBuildMethodTest () 
 		{
-			method = GetMethodForTestFrom ("Test.Rules.Smells.MainDialog", "Build");
-			messageCollection = rule.CheckMethod (method, new MinimalRunner ());
-			Assert.IsNull (messageCollection);
+			AssertRuleDoesNotApply<MainDialog> ("Build");
 		}
 
 		[Test]
 		public void DialogInitializeComponentMethodTest () 
 		{
-			method = GetMethodForTestFrom ("Test.Rules.Smells.MainDialog", "InitializeComponent");
-			messageCollection = rule.CheckMethod (method, new MinimalRunner ());
-			Assert.IsNotNull (messageCollection);
-			Assert.AreEqual (1, messageCollection.Count);
+			AssertRuleFailure<MainDialog> ("InitializeComponent", 1);
 		}
 
 		[Test]
 		public void WindowBuildMethodTest () 
 		{
-			method = GetMethodForTestFrom ("Test.Rules.Smells.MainWindow", "Build");
-			messageCollection = rule.CheckMethod (method, new MinimalRunner ());
-			Assert.IsNull (messageCollection);
+			AssertRuleDoesNotApply<MainWindow> ("Build");
 		}
 
 		[Test]
 		public void WindowInitializeComponentMethodTest () 
 		{
-			method = GetMethodForTestFrom ("Test.Rules.Smells.MainWindow", "InitializeComponent");
-			messageCollection = rule.CheckMethod (method, new MinimalRunner ());
-			Assert.IsNotNull (messageCollection);
-			Assert.AreEqual (1, messageCollection.Count);
+			AssertRuleFailure<MainWindow> ("InitializeComponent", 1);
 		}
-	
+
 		[Test]
 		public void FalseInitializeComponentTest () 
 		{
-			method = GetMethodForTest ("InitializeComponent");
-			messageCollection = rule.CheckMethod (method, new MinimalRunner ());
-			Assert.IsNotNull (messageCollection);
-			Assert.AreEqual (1, messageCollection.Count);
+			AssertRuleFailure<AvoidLongMethodsTest> ("InitializeComponent", 1);
 		}
 
 		[Test]
 		public void FormInitializeComponentTest () 
 		{
-			method = GetMethodForTestFrom ("Test.Rules.Smells.MainForm", "InitializeComponent");
-			messageCollection = rule.CheckMethod (method, new MinimalRunner ());
-			Assert.IsNull (messageCollection);
+			AssertRuleDoesNotApply<MainForm> ("InitializeComponent");
 		}
 
 		[Test]
 		public void FormBuildMethodTest () 
 		{
-			method = GetMethodForTestFrom ("Test.Rules.Smells.MainForm", "Build");
-			messageCollection = rule.CheckMethod (method, new MinimalRunner ());
-			Assert.IsNotNull (messageCollection);
-			Assert.AreEqual (1, messageCollection.Count);
+			AssertRuleFailure<MainForm> ("Build", 1);
+		}
+
+		[Test]
+		public void LongStaticConstructorWithoutFieldsTest () 
+		{
+			AssertRuleFailure<LongStaticConstructorWithoutFields> (".cctor", 1);
+		}
+		
+		[Test]
+		public void LongStaticConstructorWithFieldsTest () 
+		{
+			AssertRuleSuccess<LongStaticConstructorWithFields> (".cctor");
+		}
+
+		[Test]
+		public void LongConstructorWithoutFieldsTest ()
+		{
+			AssertRuleFailure<LongConstructorWithoutFields> (".ctor", 1);
+		}
+
+		[Test]
+		public void LongConstructorWithFieldsTest ()
+		{
+			AssertRuleSuccess<LongConstructorWithFields> (".ctor");
+		}
+
+		[Test]
+		public void LongConstructorWithReadonlyFieldsTest ()
+		{
+			AssertRuleSuccess<LongConstructorWithReadonlyFields> (".ctor");
+		}
+	}
+
+	[TestFixture]
+	public class AvoidLongMethods_IlTest : AvoidLongMethodsTest {
+
+		public AvoidLongMethods_IlTest ()
+		{
+			Rule.UseIlApproximation = true;
+		}
+	}
+
+	[TestFixture]
+	public class AvoidLongMethods_SlocTest : AvoidLongMethodsTest {
+
+		public AvoidLongMethods_SlocTest ()
+		{
+			Rule.UseIlApproximation = false;
+		}
+
+		[TestFixtureSetUp]
+		public void FixtureSetUp ()
+		{
+			AssemblyDefinition assembly = DefinitionLoader.GetAssemblyDefinition<AvoidLongMethodsTest> ();
+			if (!assembly.MainModule.HasDebuggingInformation ())
+				Assert.Ignore ("Debugging symbols non-available to compute SLOC.");
 		}
 	}
 }
