@@ -32,10 +32,9 @@ using System.Text;
 using GuiCompare;
 
 public partial class MainWindow: Gtk.Window
-{	
-	string monodir;
+{
 	InfoManager info_manager;
-	LoadCompAssembly reference_loader, target_loader;
+	Func<CompAssembly> reference_loader, target_loader;
 	CompareContext context;
 	
 	static Gdk.Pixbuf classPixbuf, delegatePixbuf, enumPixbuf;
@@ -193,7 +192,7 @@ public partial class MainWindow: Gtk.Window
 				ComparisonNode n = tree.Model.GetValue (iter, (int)TreeCol.Node) as ComparisonNode;
 				StringBuilder sb = new StringBuilder();
 
-				if (n != null) msgs = n.messages;
+				if (n != null) msgs = n.Messages;
 				if (msgs != null && msgs.Count > 0) {
 					sb.Append ("<b>Errors:</b>\n");
 
@@ -202,7 +201,7 @@ public partial class MainWindow: Gtk.Window
 					}
 				}
 				
-				if (n != null) msgs = n.todos;
+				if (n != null) msgs = n.Todos;
 				if (msgs != null && msgs.Count > 0) {
 					sb.Append ("<b>TODO:</b>\n");
 					for (int i = 0; i < msgs.Count; i ++) {
@@ -260,12 +259,12 @@ public partial class MainWindow: Gtk.Window
 		}
 	}
 	
-	public void SetTarget (LoadCompAssembly target)
+	public void SetTarget (Func<CompAssembly> target)
 	{
 		target_loader = target;
 	}
 	
-	public void SetReference (LoadCompAssembly reference)
+	public void SetReference (Func<CompAssembly> reference)
 	{
 		reference_loader = reference;
 	}
@@ -300,54 +299,61 @@ public partial class MainWindow: Gtk.Window
 		
 		// now generate new content asynchronously
 		context = new CompareContext (reference_loader, target_loader);
-		
+
 		context.ProgressChanged += delegate (object sender, CompareProgressChangedEventArgs e) {
-			/* update our progress bar */
-			Status = e.Message;
-			Progress = e.Progress;
+			Application.Invoke (delegate {
+				Status = e.Message;
+				Progress = e.Progress;
+			});
 		};
+		
 		context.Error += delegate (object sender, CompareErrorEventArgs e) {
-			Console.WriteLine ("ERROR: {0}", e.Message);
-			MessageDialog md = new MessageDialog (this, 0, MessageType.Error, ButtonsType.Ok, false,
-			                                      e.Message);
-			md.Response += delegate (object s, ResponseArgs ra) {
-				md.Hide ();
-			};
-			md.Show();
-			Status = String.Format ("Comparison failed at {0}", DateTime.Now);
-			Progress = 0.0;
-			progressbar1.Visible = false;
+			Application.Invoke (delegate {
+				Console.WriteLine ("ERROR: {0}", e.Message);
+				MessageDialog md = new MessageDialog (this, 0, MessageType.Error, ButtonsType.Ok, false,
+				                                      e.Message);
+				md.Response += delegate (object s, ResponseArgs ra) {
+					md.Hide ();
+				};
+				md.Show();
+				Status = String.Format ("Comparison failed at {0}", DateTime.Now);
+				Progress = 0.0;
+				progressbar1.Visible = false;
+			});
 		};
+
 		context.Finished += delegate (object sender, EventArgs e) {
-			DateTime finish_time = DateTime.Now;
-			
-			Status = String.Format ("Comparison completed at {0}", finish_time);
-			
-			context.Comparison.PropagateCounts ();
-			PopulateTreeFromComparison (context.Comparison);
-			Progress = 0.0;
-			done (this);
-			progressbar1.Visible = false;
-
-			CompareHistory[] history = Config.Recent[0].History;
-			
-			if (history == null || history.Length == 0 ||
-			    (history[history.Length-1].Extras != context.Comparison.Extra ||
-			     history[history.Length-1].Errors != context.Comparison.Warning ||
-			     history[history.Length-1].Missing != context.Comparison.Missing ||
-			     history[history.Length-1].Niexs != context.Comparison.Niex ||
-			     history[history.Length-1].Todos != context.Comparison.Todo)) {
-
-				CompareHistory history_entry = new CompareHistory();
-				history_entry.CompareTime = finish_time;
-				history_entry.Extras = context.Comparison.Extra;
-				history_entry.Errors = context.Comparison.Warning;
-				history_entry.Missing = context.Comparison.Missing;
-				history_entry.Niexs = context.Comparison.Niex;
-				history_entry.Todos = context.Comparison.Todo;
-				Config.Recent[0].AddHistoryEntry (history_entry);
-				Config.Save ();
-			}
+			Application.Invoke (delegate {
+				DateTime finish_time = DateTime.Now;
+				
+				Status = String.Format ("Comparison completed at {0}", finish_time);
+				
+				context.Comparison.PropagateCounts ();
+				PopulateTreeFromComparison (context.Comparison);
+				Progress = 0.0;
+				done (this);
+				progressbar1.Visible = false;
+	
+				CompareHistory[] history = Config.Recent[0].History;
+				
+				if (history == null || history.Length == 0 ||
+				    (history[history.Length-1].Extras != context.Comparison.Extra ||
+				     history[history.Length-1].Errors != context.Comparison.Warning ||
+				     history[history.Length-1].Missing != context.Comparison.Missing ||
+				     history[history.Length-1].Niexs != context.Comparison.Niex ||
+				     history[history.Length-1].Todos != context.Comparison.Todo)) {
+	
+					CompareHistory history_entry = new CompareHistory();
+					history_entry.CompareTime = finish_time;
+					history_entry.Extras = context.Comparison.Extra;
+					history_entry.Errors = context.Comparison.Warning;
+					history_entry.Missing = context.Comparison.Missing;
+					history_entry.Niexs = context.Comparison.Niex;
+					history_entry.Todos = context.Comparison.Todo;
+					Config.Recent[0].AddHistoryEntry (history_entry);
+					Config.Save ();
+				}
+			});
 		};
 		treeStore.Clear ();
 		context.Compare ();
@@ -355,7 +361,7 @@ public partial class MainWindow: Gtk.Window
 	
 	Gdk.Pixbuf TypePixbufFromComparisonNode (ComparisonNode node)
 	{
-		switch (node.type) {
+		switch (node.Type) {
 		case CompType.Assembly: return assemblyPixbuf;
 		case CompType.Namespace: return namespacePixbuf;
 		case CompType.Attribute: return attributePixbuf;
@@ -374,7 +380,7 @@ public partial class MainWindow: Gtk.Window
 	
 	Gdk.Pixbuf StatusPixbufFromComparisonNode (ComparisonNode node)
 	{
-		switch (node.status) {
+		switch (node.Status) {
 		case ComparisonStatus.None: return okPixbuf;
 		case ComparisonStatus.Missing: return missingPixbuf;
 		case ComparisonStatus.Extra: return extraPixbuf;
@@ -385,7 +391,7 @@ public partial class MainWindow: Gtk.Window
 
 	string StatusForegroundFromComparisonNode (ComparisonNode node)
 	{
-		switch (node.status) {
+		switch (node.Status) {
 		case ComparisonStatus.Missing: return "darkred";
 		case ComparisonStatus.Extra: return "green";
 		case ComparisonStatus.Error: return "red";
@@ -398,7 +404,7 @@ public partial class MainWindow: Gtk.Window
 	void PopulateTreeFromComparison (ComparisonNode root)
 	{
 		Gtk.TreeIter iter =
-			treeStore.AppendValues (root.name,
+			treeStore.AppendValues (root.Name,
 			                        TypePixbufFromComparisonNode (root),
 			                        StatusPixbufFromComparisonNode (root),
 			                        root.Missing == 0 ? null : missingPixbuf,
@@ -416,7 +422,7 @@ public partial class MainWindow: Gtk.Window
 		
 		Gtk.TreePath path = treeStore.GetPath (iter);
 		
-		foreach (ComparisonNode n in root.children) {
+		foreach (ComparisonNode n in root.Children) {
 			PopulateTreeFromComparison (iter, n);
 		}
 		
@@ -427,7 +433,7 @@ public partial class MainWindow: Gtk.Window
 	{
 		Gtk.TreeIter citer = 
 			treeStore.AppendValues (iter,
-			                        node.name,
+			                        node.Name,
 			                        TypePixbufFromComparisonNode (node),
 			                        StatusPixbufFromComparisonNode (node),
 			                        node.Missing == 0 ? null : missingPixbuf,
@@ -439,12 +445,12 @@ public partial class MainWindow: Gtk.Window
 			                        node.Todo == 0 ? null : todoPixbuf,
 			                        node.Todo == 0 ? null : String.Format (":{0}", node.Todo),
 			                        node.Niex == 0 ? null : niexPixbuf,
-			                        node.Niex == 0 ? null : ((node.Niex == 1 && node.throws_niex) ? null : String.Format (":{0}", node.Niex)),
+			                        node.Niex == 0 ? null : ((node.Niex == 1 && node.ThrowsNIE) ? null : String.Format (":{0}", node.Niex)),
 			                        node,
 			                        StatusForegroundFromComparisonNode (node));
 
 		
-		foreach (ComparisonNode n in node.children) {
+		foreach (ComparisonNode n in node.Children) {
 			PopulateTreeFromComparison (citer, n);
 		}
 	}
@@ -457,12 +463,12 @@ public partial class MainWindow: Gtk.Window
 		if (n == null)
 			return false;
 		
-		if ((ShowMissing.Active && (n.status == ComparisonStatus.Missing || n.Missing > 0)) ||
-		    (ShowExtra.Active && (n.status == ComparisonStatus.Extra || n.Extra > 0)) ||
-		    (ShowErrors.Active && (n.status == ComparisonStatus.Error || n.Warning > 0)) ||
+		if ((ShowMissing.Active && (n.Status == ComparisonStatus.Missing || n.Missing > 0)) ||
+		    (ShowExtra.Active && (n.Status == ComparisonStatus.Extra || n.Extra > 0)) ||
+		    (ShowErrors.Active && (n.Status == ComparisonStatus.Error || n.Warning > 0)) ||
 		    (ShowTodo.Active && (n.Todo > 0)) ||
 		    (ShowNotImplemented.Active && (n.Niex > 0)) ||
-		    ShowPresent.Active && n.status == ComparisonStatus.None)
+		    ShowPresent.Active && n.Status == ComparisonStatus.None)
 			
 			return true;
 		else
@@ -549,4 +555,18 @@ public partial class MainWindow: Gtk.Window
 				
 		cc.Destroy ();
 	}
+
+	protected virtual void OnToggleRowExpansion (object sender, System.EventArgs e)
+	{
+		Gtk.TreeIter iter;
+		if (tree.Selection.GetSelected (out iter)) {			
+			Gtk.TreePath path = tree.Model.GetPath(iter);
+			if (path != null) {
+				if (tree.GetRowExpanded (path))
+					tree.CollapseRow (path);
+				else
+					tree.ExpandRow (path, false);
+			}
+		}
+	}
 }
