@@ -1,14 +1,30 @@
-
-namespace GuiCompare {
 //
-// mono-api-diff.cs - Compares 2 xml files produced by mono-api-info and
-//		      produces a file suitable to build class status pages.
+// Masterinfo.cs
 //
 // Authors:
 //	Gonzalo Paniagua Javier (gonzalo@ximian.com)
 //	Marek Safar				(marek.safar@gmail.com)
 //
-// (C) 2003 Novell, Inc (http://www.novell.com)
+// (C) 2003 - 2008 Novell, Inc. (http://www.novell.com)
+//
+// Permission is hereby granted, free of charge, to any person obtaining
+// a copy of this software and associated documentation files (the
+// "Software"), to deal in the Software without restriction, including
+// without limitation the rights to use, copy, modify, merge, publish,
+// distribute, sublicense, and/or sell copies of the Software, and to
+// permit persons to whom the Software is furnished to do so, subject to
+// the following conditions:
+//
+// The above copyright notice and this permission notice shall be
+// included in all copies or substantial portions of the Software.
+//
+// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
+// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
+// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
+// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
+// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
+// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
+// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
 using System;
@@ -18,22 +34,7 @@ using System.Reflection;
 using System.Text;
 using System.Xml;
 
-
-// 		static int Main (string [] args)
-// 		{
-// 			if (args.Length != 2)
-// 				return 1;
-
-// 			XMLAssembly ms = CreateXMLAssembly (args [0]);
-// 			XMLAssembly mono = CreateXMLAssembly (args [1]);
-// 			XmlDocument doc = ms.CompareAndGetDocument (mono);
-
-// 			XmlTextWriter writer = new XmlTextWriter (Console.Out);
-// 			writer.Formatting = Formatting.Indented;
-// 			doc.WriteTo (writer);
-
-// 			return 0;
-// 		}
+namespace GuiCompare {
 
 	public class Counters
 	{
@@ -353,7 +354,7 @@ using System.Xml;
 		string layout;
 		public XMLAttributes attributes;
 		public XMLInterfaces interfaces;
-		XMLGenericTypeConstraints genericConstraints;
+		XMLGenericParameters genericParameters;
 		public XMLFields fields;
 		public XMLConstructors constructors;
 		public XMLProperties properties;
@@ -407,9 +408,9 @@ using System.Xml;
 				child = child.NextSibling;
 			}
 
-			if (child != null && child.Name == "generic-type-constraints") {
-				genericConstraints = new XMLGenericTypeConstraints ();
-				genericConstraints.LoadData (child);
+			if (child != null && child.Name == "generic-parameters") {
+				genericParameters = new XMLGenericParameters ();
+				genericParameters.LoadData (child);
 				child = child.NextSibling;
 			}
 
@@ -700,35 +701,39 @@ using System.Xml;
 		}
 	}
 
-	public abstract class XMLGenericGroup : XMLNameGroup
+	public class XMLGenericParameters : XMLNameGroup
 	{
 		string attributes;
+		XMLGenericParameterConstraints constraints;
+
+		public override string GroupName {
+			get { return "generic-parameters"; }
+		}
+
+		public override string Name {
+			get { return "generic-parameters"; }
+		}
 
 		protected override void LoadExtraData (string name, XmlNode node)
 		{
-			attributes = ((XmlElement) node).GetAttribute ("generic-attribute");
+			attributes = ((XmlElement) node).GetAttribute ("attributes");
+
+			var child = node.FirstChild;
+			if (child != null && child.Name == "generic-parameter-constraints") {
+				constraints = new XMLGenericParameterConstraints ();
+				constraints.LoadData (child);
+			}
 		}
 	}
 
-	public class XMLGenericTypeConstraints : XMLGenericGroup
+	public class XMLGenericParameterConstraints : XMLNameGroup
 	{
 		public override string GroupName {
-			get { return "generic-type-constraints"; }
+			get { return "generic-parameter-constraints"; }
 		}
 
 		public override string Name {
-			get { return "generic-type-constraint"; }
-		}
-	}
-
-	public class XMLGenericMethodConstraints : XMLGenericGroup
-	{
-		public override string GroupName {
-			get { return "generic-method-constraints"; }
-		}
-
-		public override string Name {
-			get { return "generic-method-constraint"; }
+			get { return "generic-parameter-constraint"; }
 		}
 	}
 
@@ -925,7 +930,7 @@ using System.Xml;
 	{
 		public Hashtable returnTypes;
 		public Hashtable parameters;
-		public Hashtable genericConstraints;
+		public Hashtable genericParameters;
 		public Hashtable signatureFlags;
 
 		[Flags]
@@ -974,13 +979,14 @@ using System.Xml;
 				parameters[name] = parms;
 			}
 
-			XmlNode genericNode = node.SelectSingleNode ("generic-method-constraints");
+			XmlNode genericNode = node.SelectSingleNode ("generic-parameters");
 			if (genericNode != null) {
-				if (genericConstraints == null)
-					genericConstraints = new Hashtable ();
-				XMLGenericMethodConstraints csts = new XMLGenericMethodConstraints ();
-				csts.LoadData (genericNode);
-				genericConstraints [name] = csts;
+				if (genericParameters == null)
+					genericParameters = new Hashtable ();
+
+				XMLGenericParameters gparams = new XMLGenericParameters ();
+				gparams.LoadData (genericNode);
+				genericParameters [name] = gparams;
 			}
 
 			base.LoadExtraData (name, node);
@@ -1009,7 +1015,7 @@ using System.Xml;
 			// ignore ReservedMasks
 			ma &= ~ MethodAttributes.ReservedMask;
 			ma &= ~ MethodAttributes.VtableLayoutMask;
-			if ((ma & MethodAttributes.FamORAssem) != 0)
+			if ((ma & MethodAttributes.FamORAssem) == MethodAttributes.FamORAssem)
 				ma = (ma & ~ MethodAttributes.FamORAssem) | MethodAttributes.Family;
 
 			// ignore the HasSecurity attribute for now
